# -*- coding: utf-8 -*-

class E2EEKeyBackupService(object):
    EKBS_REQ_TYPE = 4
    EKBS_RES_TYPE = 4
    
    def __init__(self):
        pass

    def createE2EEKeyBackup(self, blobHeader: str, blobPayload: str, reason: int):
        """
        - reason
            UNKNOWN(0),
            BACKGROUND_NEW_KEY_CREATED(1),
            BACKGROUND_PERIODICAL_VERIFICATION(2),
            FOREGROUND_NEW_PIN_REGISTERED(3),
            FOREGROUND_VERIFICATION(4);
        """
        params = [
            [12, 2, [
                [11, 1, blobHeader],
                [11, 2, blobPayload],
                [8, 3, reason]
            ]]
        ]
        sqrd = self.generateDummyProtocol('createE2EEKeyBackup', params, self.EKBS_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_E2EE_KEY_BACKUP_ENDPOINT, sqrd, self.EKBS_RES_TYPE)

    def getE2EEKeyBackupCertificates(self):
        params = [
            [12, 2, []]
        ]
        sqrd = self.generateDummyProtocol('getE2EEKeyBackupCertificates', params, self.EKBS_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_E2EE_KEY_BACKUP_ENDPOINT, sqrd, self.EKBS_RES_TYPE)

    def getE2EEKeyBackupInfo(self):
        params = [
            [12, 2, []]
        ]
        sqrd = self.generateDummyProtocol('getE2EEKeyBackupInfo', params, self.EKBS_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_E2EE_KEY_BACKUP_ENDPOINT, sqrd, self.EKBS_RES_TYPE)


# -*- coding: utf-8 -*-

class BaseEvent:

    def onReady(self):
        pass

    def onPing(self):
        pass

    def onError(self):
        pass

    def onStop(self):
        print("bye")

class HookEvents(BaseEvent):

    def __init__(self):
        pass

    def onInitializePushConn(self):
        pass


# -*- coding: utf-8 -*-
import json
import time
import os
import qrcode
import re
import requests
import abc

from .exceptions import LineServiceException


class Helpers(object, metaclass=abc.ABCMeta):
    def __init__(self):
        self.liff_token_cache = {}

    @abc.abstractmethod
    def getSavePath(self, dirname: str):
        return NotImplemented

    def squareMemberIdIsMe(self, squareMemberId):
        if self.can_use_square:
            if squareMemberId in self.squares.get(2, {}).keys():
                return True
            else:
                return False
        else:
            raise Exception("Not support Square")

    def sendLiff(
        self,
        to,
        messages,
        tryConsent=True,
        forceIssue=False,
        liffId="1562242036-RW04okm",
    ):
        cache_key = f"{to}"
        use_cache = False
        if cache_key not in self.liff_token_cache or forceIssue:
            try:
                liff = self.issueLiffView(to, liffId)
            except LineServiceException as e:
                self.log(f"[sendLiff] issueLiffView error: {e}")
                if e.code == 3 and tryConsent:
                    payload = e.metadata
                    consentRequired = self.checkAndGetValue(
                        payload, "consentRequired", 3
                    )
                    channelId = self.checkAndGetValue(consentRequired, "channelId", 1)
                    consentUrl = self.checkAndGetValue(consentRequired, "consentUrl", 2)
                    toType = self.getToType(to)
                    hasConsent = False
                    if toType == 4:
                        hasConsent = self.tryConsentAuthorize(consentUrl)
                    else:
                        hasConsent = self.tryConsentLiff(channelId)
                    if hasConsent:
                        return self.sendLiff(
                            to, messages, tryConsent=False, liffId=liffId
                        )
                raise Exception(f"Failed to send Liff: {to}")
            except Exception as e:
                return e
            token = self.checkAndGetValue(liff, "accessToken", 3)
            self.log(f"[sendLiff] issue new token for {cache_key}...")
        else:
            token = self.liff_token_cache[cache_key]
            use_cache = True
            self.log(f"[sendLiff] using cache token for {cache_key}", True)
        liff_headers = {
            "Accept": "application/json, text/plain, */*",
            "User-Agent": "Mozilla/5.0 (Linux; Android 4.4.2; G730-U00 Build/JLS36C) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/30.0.0.0 Mobile Safari/537.36 Line/9.8.0",
            "Accept-Encoding": "gzip, deflate",
            "content-Type": "application/json",
            "X-Requested-With": "jp.naver.line.android",
        }
        liff_headers["authorization"] = "Bearer %s" % (token)
        if type(messages) == list:
            messages = {"messages": messages}
        else:
            messages = {"messages": [messages]}
        resp = self.server.postContent(
            "https://api.line.me/message/v3/share",
            headers=liff_headers,
            data=json.dumps(messages),
        )
        if resp.status_code == 200:
            self.liff_token_cache[cache_key] = token
        elif use_cache:
            return self.sendLiff(to, messages, False, True, liffId)
        return resp.text

    def tryConsentLiff(self, channelId, on=None, referer=None):
        if on is None:
            on = ["P", "CM"]
        payload = {"on": on, "off": []}
        data = json.dumps(payload)
        hr = {
            "X-LINE-ChannelId": str(channelId),
            "X-LINE-Access": self.authToken,
            "User-Agent": "Mozilla/5.0 (Linux; Android 8.0.1; SAMSUNG Realise/DeachSword; wv) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/56.0.2924.87 Mobile Safari/537.36",
            "Content-Type": "application/json",
            "X-Line-Application": self.APP_NAME,
            "X-Requested-With": "XMLHttpRequest",
            "Accept-Language": "zh-TW,en-US;q=0.8",
        }
        if referer is not None:
            hr["referer"] = referer
        r = self.server.postContent(
            "https://access.line.me/dialog/api/permissions", data=data, headers=hr
        )
        if r.status_code == 200:
            return True
        print(f"tryConsentLiff failed: {r.status_code}")
        return False

    def tryConsentAuthorize(
        self, consentUrl, allPermission=None, approvedPermission=None
    ):
        if allPermission is None:
            allPermission = ["P", "CM"]
        if approvedPermission is None:
            approvedPermission = ["P", "CM"]
        CHANNEL_ID = None
        CSRF_TOKEN = None
        session = requests.Session()
        hr = {
            "X-LINE-Access": self.authToken,
            "User-Agent": "Mozilla/5.0 (Linux; Android 8.0.1; SAMSUNG Realise/DeachSword; wv) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/56.0.2924.87 Mobile Safari/537.36",
            "X-Line-Application": self.APP_NAME,
        }
        r = session.get(consentUrl, headers=hr)
        if r.status_code == 200:
            resp = r.text
            # GET CSRF TOKEN
            CHANNEL_ID = re.findall(self.CONSENT_CHANNEL_ID_REGEX, resp)[0]
            CSRF_TOKEN = re.findall(self.CONSENT_CSRF_TOKEN_REGEX, resp)[0]
            self.log(f"CHANNEL_ID: {CHANNEL_ID}")
            self.log(f"CSRF_TOKEN: {CSRF_TOKEN}")
        if CHANNEL_ID and CSRF_TOKEN:
            url = "https://access.line.me/oauth2/v2.1/authorize/consent"
            payload = {
                "allPermission": allPermission,
                "approvedPermission": allPermission,
                "channelId": CHANNEL_ID,
                "__csrf": CSRF_TOKEN,
                "__WLS": "",
                "allow": True,
            }
            r = session.post(url, data=payload, headers=hr)
            if r.status_code == 200:
                return True
            print(f"tryConsentAuthorize failed: {r.status_code}")
        else:
            raise Exception(
                f"tryConsentAuthorize failed: STATUS_CODE: {r.status_code}, CHANNEL_ID: {CHANNEL_ID}, CSRF_TOKEN: {CSRF_TOKEN}"
            )
        return False

    def getToType(self, mid):
        """
        USER(0),
        ROOM(1),
        GROUP(2),
        SQUARE(3),
        SQUARE_CHAT(4),
        SQUARE_MEMBER(5),
        BOT(6);
        """
        _u = mid[0]
        if _u == "u":
            return 0
        if _u == "r":
            return 1
        if _u == "c":
            return 2
        if _u == "s":
            return 3
        if _u == "m":
            return 4
        if _u == "p":
            return 5
        if _u == "v":
            return 6

    def getAccessToken(self, client_id, redirect_uri, otp, code):
        data = {
            "client_id": str(client_id),  # channel id
            # intent://result#Intent;package=xxx;scheme=lineauth;end",
            "redirect_uri": redirect_uri,
            "otp": otp,  # len 20
            "code": code,  # len 20
            "grant_type": "authorization_code",
        }
        hr = {
            "Content-Type": "application/x-www-form-urlencoded",
            "User-Agent": "Mozilla/5.0 (Linux; Android 8.0.1; SAMSUNG Realise/DeachSword; wv) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/56.0.2924.87 Mobile Safari/537.36",
        }
        r = self.server.postContent(
            "https://access.line.me/v2/oauth/accessToken", data=data, headers=hr
        )
        return r.json["access_token"]

    def checkRespIsSuccessWithLpv(self, resp, lpv: int = 1, status_code: int = 200):
        ckStatusCode = lpv != 1
        if lpv == 1:
            if "x-lc" in resp.headers:
                if resp.headers["x-lc"] != str(status_code):
                    return False
            else:
                ckStatusCode = True
        if ckStatusCode:
            if resp.status_code != status_code:
                return False
        return True

    def checkIsVideo(self, filename: str):
        video_suffix = [".mp4", ".mkv", ".webm"]
        for _vs in video_suffix:
            if filename.endswith(_vs):
                return True
        return False

    def getProfileCoverObjIdAndUrl(self, mid: str):
        video_obj = None
        video_url = None
        detail = self.getProfileCoverDetail(mid)["result"]
        coverObsInfo = self.checkAndGetValue(
            detail, "coverObsInfo"
        )  # detail['coverObsInfo']
        videoCoverObsInfo = self.checkAndGetValue(
            detail, "videoCoverObsInfo"
        )  # detail['videoCoverObsInfo']
        url = (
            self.LINE_OBS_DOMAIN
            + f'/r/{coverObsInfo["serviceName"]}/{coverObsInfo["obsNamespace"]}/{coverObsInfo["objectId"]}'
        )
        if videoCoverObsInfo is not None:
            video_obj = videoCoverObsInfo["objectId"]
            video_url = (
                self.LINE_OBS_DOMAIN
                + f'/r/{videoCoverObsInfo["serviceName"]}/{videoCoverObsInfo["obsNamespace"]}/{videoCoverObsInfo["objectId"]}'
            )
        return url, video_url, coverObsInfo["objectId"], video_obj

    def getProfilePictureObjIdAndUrl(self, mid: str):
        url = None
        url_video = None
        objectId = mid
        objectId_video = None
        serviceName = "talk"
        obsNamespace = None
        midType = self.getToType(mid)
        if midType == 0:
            obsNamespace = "p"
            objectId_video = f"{mid}/vp"
            # vp.sjpg, vp.small
        elif midType in [1, 2]:
            obsNamespace = "g"
        elif midType in [3, 4, 5]:
            serviceName = "g2"
            obsNamespace = "group"
            if midType == 5:
                obsNamespace = "member"
        else:
            raise ValueError(f"Not support midType: {midType}")
        url = self.LINE_OBS_DOMAIN + f"/r/{serviceName}/{obsNamespace}/{objectId}"
        if objectId_video is not None:
            url_video = (
                self.LINE_OBS_DOMAIN
                + f"/r/{serviceName}/{obsNamespace}/{objectId_video}"
            )
        return url, url_video, objectId, objectId_video

    def checkAndGetValue(self, value, *args):
        for arg in args:
            if type(value) == dict:
                if arg in value:
                    return value[arg]
            else:
                data = getattr(value, str(arg), None)
                if data is not None:
                    return data
                if isinstance(arg, int):
                    data = getattr(value, f"val_{arg}", None)
                    if data is not None:
                        return data
        return None

    def checkAndSetValue(self, value, *args):
        set = args[-1]
        args = args[:-1]
        if not args:
            raise ValueError(f"Invalid arguments: {args}")
        for arg in args:
            if type(value) == dict:
                value[arg] = set
            else:
                setattr(value, str(arg), set)
        return value

    def genQrcodeImageAndPrint(
        self, url: str, filename: str = None, output_char: list = None
    ):
        if output_char is None:
            output_char = ["　", "■"]
        if filename is None:
            filename = str(time.time())
        savePath = self.getSavePath(".images")
        savePath = savePath + f"/qr_{filename}.png"
        qr = qrcode.QRCode(
            version=1,
            error_correction=qrcode.constants.ERROR_CORRECT_L,
        )
        qr.add_data(url)
        qr.make()

        def win_qr_make(x):
            return print(
                "".join(
                    [output_char[1]]
                    + [output_char[0] if y is True else output_char[1] for y in x]
                    + [output_char[1]]
                )
            )

        # 如果你問我問啥要加這段, 我可以告訴你fk u line >:( 你可以看到qr外面有一圈, 這是讓line讀到的必要條件(啊明明就可以不用 line的判定有夠爛)
        fixed_bored = [False for _b in range(len(qr.modules[0]))]
        for qr_module in [fixed_bored] + qr.modules + [fixed_bored]:
            win_qr_make(qr_module)
        img = qr.make_image()
        img.save(savePath)
        return savePath

    def sendMention(self, to, text="", mids=[], prefix=True):
        if type(mids) != list:
            mids = [mids]
        tag = "@chrline"
        str_tag = "@!"
        arr_data = []
        if mids == []:
            raise ValueError(f"Invalid mids: {mids}")
        if str_tag not in text:
            message = text if prefix else ""
            for mid in mids:
                slen = len(message)
                elen = len(message) + len(tag)
                arr = {"S": str(slen), "E": str(elen), "M": mid}
                arr_data.append(arr)
                message += tag
            if not prefix:
                message += text
        else:
            if text.count(str_tag) != len(mids):
                raise ValueError(
                    f"Invalid tag length: {text.count(str_tag)}/{len(mids)}"
                )
            text_data = text.split(str_tag)
            message = ""
            for mid in mids:
                message += str(text_data[mids.index(mid)])
                slen = len(message)
                elen = len(message) + len(tag)
                arr = {"S": str(slen), "E": str(elen), "M": mid}
                arr_data.append(arr)
                message += tag
            message += text_data[-1]
        return self.sendMessage(
            to,
            message,
            contentMetadata={
                "MENTION": str('{"MENTIONEES":' + json.dumps(arr_data) + "}")
            },
        )

    def getMentioneesByMsgData(self, msg: dict):
        a = []
        b = self.checkAndGetValue(msg, "contentMetadata", 18)
        if b is not None:
            if "MENTION" in b:
                c = json.loads(b["MENTION"])
                print(c)
                for _m in c["MENTIONEES"]:
                    print(_m["M"])
                    a.append(_m["M"])
        return a

    def genMentionData(self, mentions: dict):
        """
        - mentions:
            - S: index
            - L: len
            - M: mid
            - A: ALL
        """
        if mentions is None or len(mentions) == 0:
            return None
        a = []
        for b in mentions:
            c = {}
            d = False
            if "A" in b:
                d = True
            c["S"] = str(b["S"])
            c["E"] = str(b["S"] + b["L"])
            if d:
                c["A"] = str(1)
            else:
                c["M"] = str(b["M"])
            a.append(c)
        a = {"MENTIONEES": a}
        return {"MENTION": json.dumps(a)}



#
# Autogenerated by Thrift Compiler (0.16.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py
#

from thrift.Thrift import TType, TMessageType, TFrozenDict, TException, TApplicationException
from thrift.protocol.TProtocol import TProtocolException
from thrift.TRecursive import fix_spec

import sys
import logging
from .ttypes import *
from thrift.Thrift import TProcessor
from thrift.transport import TTransport
all_structs = []


class Iface(object):
    def sendMessage(self):
        pass

    def unsendSquareMessage(self):
        pass

    def fetchMyEvents(self):
        pass

    def getSquareEmid(self):
        pass

    def getSquareMembersBySquare(self):
        pass

    def manualRepair(self):
        pass

    def inviteIntoSquareChat(self):
        pass

    def inviteToSquare(self):
        pass

    def getJoinedSquares(self):
        pass

    def markAsRead(self):
        pass

    def reactToMessage(self):
        pass

    def findSquareByInvitationTicket(self):
        pass

    def fetchSquareChatEvents(self):
        pass

    def getSquare(self):
        pass

    def getJoinableSquareChats(self):
        pass

    def createSquare(self):
        pass

    def getSquareChatAnnouncements(self):
        pass


class Client(Iface):
    def __init__(self, iprot, oprot=None):
        self._iprot = self._oprot = iprot
        if oprot is not None:
            self._oprot = oprot
        self._seqid = 0

    def sendMessage(self):
        self.send_sendMessage()
        return self.recv_sendMessage()

    def send_sendMessage(self):
        self._oprot.writeMessageBegin('sendMessage', TMessageType.CALL, self._seqid)
        args = sendMessage_args()
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_sendMessage(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = sendMessage_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        if result.e is not None:
            raise result.e
        raise TApplicationException(TApplicationException.MISSING_RESULT, "sendMessage failed: unknown result")

    def unsendSquareMessage(self):
        self.send_unsendSquareMessage()
        return self.recv_unsendSquareMessage()

    def send_unsendSquareMessage(self):
        self._oprot.writeMessageBegin('unsendSquareMessage', TMessageType.CALL, self._seqid)
        args = unsendSquareMessage_args()
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_unsendSquareMessage(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = unsendSquareMessage_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        if result.e is not None:
            raise result.e
        raise TApplicationException(TApplicationException.MISSING_RESULT, "unsendSquareMessage failed: unknown result")

    def fetchMyEvents(self):
        self.send_fetchMyEvents()
        return self.recv_fetchMyEvents()

    def send_fetchMyEvents(self):
        self._oprot.writeMessageBegin('fetchMyEvents', TMessageType.CALL, self._seqid)
        args = fetchMyEvents_args()
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_fetchMyEvents(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = fetchMyEvents_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        if result.e is not None:
            raise result.e
        raise TApplicationException(TApplicationException.MISSING_RESULT, "fetchMyEvents failed: unknown result")

    def getSquareEmid(self):
        self.send_getSquareEmid()
        return self.recv_getSquareEmid()

    def send_getSquareEmid(self):
        self._oprot.writeMessageBegin('getSquareEmid', TMessageType.CALL, self._seqid)
        args = getSquareEmid_args()
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_getSquareEmid(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = getSquareEmid_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        if result.e is not None:
            raise result.e
        raise TApplicationException(TApplicationException.MISSING_RESULT, "getSquareEmid failed: unknown result")

    def getSquareMembersBySquare(self):
        self.send_getSquareMembersBySquare()
        return self.recv_getSquareMembersBySquare()

    def send_getSquareMembersBySquare(self):
        self._oprot.writeMessageBegin('getSquareMembersBySquare', TMessageType.CALL, self._seqid)
        args = getSquareMembersBySquare_args()
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_getSquareMembersBySquare(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = getSquareMembersBySquare_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        if result.e is not None:
            raise result.e
        raise TApplicationException(TApplicationException.MISSING_RESULT, "getSquareMembersBySquare failed: unknown result")

    def manualRepair(self):
        self.send_manualRepair()
        return self.recv_manualRepair()

    def send_manualRepair(self):
        self._oprot.writeMessageBegin('manualRepair', TMessageType.CALL, self._seqid)
        args = manualRepair_args()
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_manualRepair(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = manualRepair_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        if result.e is not None:
            raise result.e
        raise TApplicationException(TApplicationException.MISSING_RESULT, "manualRepair failed: unknown result")

    def inviteIntoSquareChat(self):
        self.send_inviteIntoSquareChat()
        return self.recv_inviteIntoSquareChat()

    def send_inviteIntoSquareChat(self):
        self._oprot.writeMessageBegin('inviteIntoSquareChat', TMessageType.CALL, self._seqid)
        args = inviteIntoSquareChat_args()
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_inviteIntoSquareChat(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = inviteIntoSquareChat_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        if result.e is not None:
            raise result.e
        raise TApplicationException(TApplicationException.MISSING_RESULT, "inviteIntoSquareChat failed: unknown result")

    def inviteToSquare(self):
        self.send_inviteToSquare()
        return self.recv_inviteToSquare()

    def send_inviteToSquare(self):
        self._oprot.writeMessageBegin('inviteToSquare', TMessageType.CALL, self._seqid)
        args = inviteToSquare_args()
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_inviteToSquare(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = inviteToSquare_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        if result.e is not None:
            raise result.e
        raise TApplicationException(TApplicationException.MISSING_RESULT, "inviteToSquare failed: unknown result")

    def getJoinedSquares(self):
        self.send_getJoinedSquares()
        return self.recv_getJoinedSquares()

    def send_getJoinedSquares(self):
        self._oprot.writeMessageBegin('getJoinedSquares', TMessageType.CALL, self._seqid)
        args = getJoinedSquares_args()
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_getJoinedSquares(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = getJoinedSquares_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        if result.e is not None:
            raise result.e
        raise TApplicationException(TApplicationException.MISSING_RESULT, "getJoinedSquares failed: unknown result")

    def markAsRead(self):
        self.send_markAsRead()
        return self.recv_markAsRead()

    def send_markAsRead(self):
        self._oprot.writeMessageBegin('markAsRead', TMessageType.CALL, self._seqid)
        args = markAsRead_args()
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_markAsRead(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = markAsRead_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        if result.e is not None:
            raise result.e
        raise TApplicationException(TApplicationException.MISSING_RESULT, "markAsRead failed: unknown result")

    def reactToMessage(self):
        self.send_reactToMessage()
        return self.recv_reactToMessage()

    def send_reactToMessage(self):
        self._oprot.writeMessageBegin('reactToMessage', TMessageType.CALL, self._seqid)
        args = reactToMessage_args()
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_reactToMessage(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = reactToMessage_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        if result.e is not None:
            raise result.e
        raise TApplicationException(TApplicationException.MISSING_RESULT, "reactToMessage failed: unknown result")

    def findSquareByInvitationTicket(self):
        self.send_findSquareByInvitationTicket()
        return self.recv_findSquareByInvitationTicket()

    def send_findSquareByInvitationTicket(self):
        self._oprot.writeMessageBegin('findSquareByInvitationTicket', TMessageType.CALL, self._seqid)
        args = findSquareByInvitationTicket_args()
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_findSquareByInvitationTicket(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = findSquareByInvitationTicket_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        if result.e is not None:
            raise result.e
        raise TApplicationException(TApplicationException.MISSING_RESULT, "findSquareByInvitationTicket failed: unknown result")

    def fetchSquareChatEvents(self):
        self.send_fetchSquareChatEvents()
        return self.recv_fetchSquareChatEvents()

    def send_fetchSquareChatEvents(self):
        self._oprot.writeMessageBegin('fetchSquareChatEvents', TMessageType.CALL, self._seqid)
        args = fetchSquareChatEvents_args()
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_fetchSquareChatEvents(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = fetchSquareChatEvents_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        if result.e is not None:
            raise result.e
        raise TApplicationException(TApplicationException.MISSING_RESULT, "fetchSquareChatEvents failed: unknown result")

    def getSquare(self):
        self.send_getSquare()
        return self.recv_getSquare()

    def send_getSquare(self):
        self._oprot.writeMessageBegin('getSquare', TMessageType.CALL, self._seqid)
        args = getSquare_args()
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_getSquare(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = getSquare_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        if result.e is not None:
            raise result.e
        raise TApplicationException(TApplicationException.MISSING_RESULT, "getSquare failed: unknown result")

    def getJoinableSquareChats(self):
        self.send_getJoinableSquareChats()
        return self.recv_getJoinableSquareChats()

    def send_getJoinableSquareChats(self):
        self._oprot.writeMessageBegin('getJoinableSquareChats', TMessageType.CALL, self._seqid)
        args = getJoinableSquareChats_args()
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_getJoinableSquareChats(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = getJoinableSquareChats_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        if result.e is not None:
            raise result.e
        raise TApplicationException(TApplicationException.MISSING_RESULT, "getJoinableSquareChats failed: unknown result")

    def createSquare(self):
        self.send_createSquare()
        return self.recv_createSquare()

    def send_createSquare(self):
        self._oprot.writeMessageBegin('createSquare', TMessageType.CALL, self._seqid)
        args = createSquare_args()
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_createSquare(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = createSquare_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        if result.e is not None:
            raise result.e
        raise TApplicationException(TApplicationException.MISSING_RESULT, "createSquare failed: unknown result")

    def getSquareChatAnnouncements(self):
        self.send_getSquareChatAnnouncements()
        return self.recv_getSquareChatAnnouncements()

    def send_getSquareChatAnnouncements(self):
        self._oprot.writeMessageBegin('getSquareChatAnnouncements', TMessageType.CALL, self._seqid)
        args = getSquareChatAnnouncements_args()
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_getSquareChatAnnouncements(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = getSquareChatAnnouncements_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        if result.e is not None:
            raise result.e
        raise TApplicationException(TApplicationException.MISSING_RESULT, "getSquareChatAnnouncements failed: unknown result")


class Processor(Iface, TProcessor):
    def __init__(self, handler):
        self._handler = handler
        self._processMap = {}
        self._processMap["sendMessage"] = Processor.process_sendMessage
        self._processMap["unsendSquareMessage"] = Processor.process_unsendSquareMessage
        self._processMap["fetchMyEvents"] = Processor.process_fetchMyEvents
        self._processMap["getSquareEmid"] = Processor.process_getSquareEmid
        self._processMap["getSquareMembersBySquare"] = Processor.process_getSquareMembersBySquare
        self._processMap["manualRepair"] = Processor.process_manualRepair
        self._processMap["inviteIntoSquareChat"] = Processor.process_inviteIntoSquareChat
        self._processMap["inviteToSquare"] = Processor.process_inviteToSquare
        self._processMap["getJoinedSquares"] = Processor.process_getJoinedSquares
        self._processMap["markAsRead"] = Processor.process_markAsRead
        self._processMap["reactToMessage"] = Processor.process_reactToMessage
        self._processMap["findSquareByInvitationTicket"] = Processor.process_findSquareByInvitationTicket
        self._processMap["fetchSquareChatEvents"] = Processor.process_fetchSquareChatEvents
        self._processMap["getSquare"] = Processor.process_getSquare
        self._processMap["getJoinableSquareChats"] = Processor.process_getJoinableSquareChats
        self._processMap["createSquare"] = Processor.process_createSquare
        self._processMap["getSquareChatAnnouncements"] = Processor.process_getSquareChatAnnouncements
        self._on_message_begin = None

    def on_message_begin(self, func):
        self._on_message_begin = func

    def process(self, iprot, oprot):
        (name, type, seqid) = iprot.readMessageBegin()
        if self._on_message_begin:
            self._on_message_begin(name, type, seqid)
        if name not in self._processMap:
            iprot.skip(TType.STRUCT)
            iprot.readMessageEnd()
            x = TApplicationException(TApplicationException.UNKNOWN_METHOD, 'Unknown function %s' % (name))
            oprot.writeMessageBegin(name, TMessageType.EXCEPTION, seqid)
            x.write(oprot)
            oprot.writeMessageEnd()
            oprot.trans.flush()
            return
        else:
            self._processMap[name](self, seqid, iprot, oprot)
        return True

    def process_sendMessage(self, seqid, iprot, oprot):
        args = sendMessage_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = sendMessage_result()
        try:
            result.success = self._handler.sendMessage()
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except SquareException as e:
            msg_type = TMessageType.REPLY
            result.e = e
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("sendMessage", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_unsendSquareMessage(self, seqid, iprot, oprot):
        args = unsendSquareMessage_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = unsendSquareMessage_result()
        try:
            result.success = self._handler.unsendSquareMessage()
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except SquareException as e:
            msg_type = TMessageType.REPLY
            result.e = e
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("unsendSquareMessage", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_fetchMyEvents(self, seqid, iprot, oprot):
        args = fetchMyEvents_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = fetchMyEvents_result()
        try:
            result.success = self._handler.fetchMyEvents()
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except SquareException as e:
            msg_type = TMessageType.REPLY
            result.e = e
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("fetchMyEvents", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_getSquareEmid(self, seqid, iprot, oprot):
        args = getSquareEmid_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = getSquareEmid_result()
        try:
            result.success = self._handler.getSquareEmid()
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except SquareException as e:
            msg_type = TMessageType.REPLY
            result.e = e
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("getSquareEmid", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_getSquareMembersBySquare(self, seqid, iprot, oprot):
        args = getSquareMembersBySquare_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = getSquareMembersBySquare_result()
        try:
            result.success = self._handler.getSquareMembersBySquare()
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except SquareException as e:
            msg_type = TMessageType.REPLY
            result.e = e
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("getSquareMembersBySquare", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_manualRepair(self, seqid, iprot, oprot):
        args = manualRepair_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = manualRepair_result()
        try:
            result.success = self._handler.manualRepair()
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except SquareException as e:
            msg_type = TMessageType.REPLY
            result.e = e
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("manualRepair", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_inviteIntoSquareChat(self, seqid, iprot, oprot):
        args = inviteIntoSquareChat_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = inviteIntoSquareChat_result()
        try:
            result.success = self._handler.inviteIntoSquareChat()
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except SquareException as e:
            msg_type = TMessageType.REPLY
            result.e = e
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("inviteIntoSquareChat", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_inviteToSquare(self, seqid, iprot, oprot):
        args = inviteToSquare_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = inviteToSquare_result()
        try:
            result.success = self._handler.inviteToSquare()
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except SquareException as e:
            msg_type = TMessageType.REPLY
            result.e = e
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("inviteToSquare", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_getJoinedSquares(self, seqid, iprot, oprot):
        args = getJoinedSquares_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = getJoinedSquares_result()
        try:
            result.success = self._handler.getJoinedSquares()
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except SquareException as e:
            msg_type = TMessageType.REPLY
            result.e = e
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("getJoinedSquares", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_markAsRead(self, seqid, iprot, oprot):
        args = markAsRead_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = markAsRead_result()
        try:
            result.success = self._handler.markAsRead()
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except SquareException as e:
            msg_type = TMessageType.REPLY
            result.e = e
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("markAsRead", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_reactToMessage(self, seqid, iprot, oprot):
        args = reactToMessage_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = reactToMessage_result()
        try:
            result.success = self._handler.reactToMessage()
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except SquareException as e:
            msg_type = TMessageType.REPLY
            result.e = e
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("reactToMessage", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_findSquareByInvitationTicket(self, seqid, iprot, oprot):
        args = findSquareByInvitationTicket_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = findSquareByInvitationTicket_result()
        try:
            result.success = self._handler.findSquareByInvitationTicket()
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except SquareException as e:
            msg_type = TMessageType.REPLY
            result.e = e
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("findSquareByInvitationTicket", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_fetchSquareChatEvents(self, seqid, iprot, oprot):
        args = fetchSquareChatEvents_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = fetchSquareChatEvents_result()
        try:
            result.success = self._handler.fetchSquareChatEvents()
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except SquareException as e:
            msg_type = TMessageType.REPLY
            result.e = e
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("fetchSquareChatEvents", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_getSquare(self, seqid, iprot, oprot):
        args = getSquare_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = getSquare_result()
        try:
            result.success = self._handler.getSquare()
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except SquareException as e:
            msg_type = TMessageType.REPLY
            result.e = e
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("getSquare", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_getJoinableSquareChats(self, seqid, iprot, oprot):
        args = getJoinableSquareChats_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = getJoinableSquareChats_result()
        try:
            result.success = self._handler.getJoinableSquareChats()
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except SquareException as e:
            msg_type = TMessageType.REPLY
            result.e = e
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("getJoinableSquareChats", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_createSquare(self, seqid, iprot, oprot):
        args = createSquare_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = createSquare_result()
        try:
            result.success = self._handler.createSquare()
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except SquareException as e:
            msg_type = TMessageType.REPLY
            result.e = e
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("createSquare", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_getSquareChatAnnouncements(self, seqid, iprot, oprot):
        args = getSquareChatAnnouncements_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = getSquareChatAnnouncements_result()
        try:
            result.success = self._handler.getSquareChatAnnouncements()
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except SquareException as e:
            msg_type = TMessageType.REPLY
            result.e = e
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("getSquareChatAnnouncements", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

# HELPER FUNCTIONS AND STRUCTURES


class sendMessage_args(object):


    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('sendMessage_args')
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(sendMessage_args)
sendMessage_args.thrift_spec = (
)


class sendMessage_result(object):
    """
    Attributes:
     - success
     - e

    """


    def __init__(self, success=None, e=None,):
        self.success = success
        self.e = e

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.STRUCT:
                    self.success = SendMessageResponse()
                    self.success.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 1:
                if ftype == TType.STRUCT:
                    self.e = SquareException.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('sendMessage_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.STRUCT, 0)
            self.success.write(oprot)
            oprot.writeFieldEnd()
        if self.e is not None:
            oprot.writeFieldBegin('e', TType.STRUCT, 1)
            self.e.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(sendMessage_result)
sendMessage_result.thrift_spec = (
    (0, TType.STRUCT, 'success', [SendMessageResponse, None], None, ),  # 0
    (1, TType.STRUCT, 'e', [SquareException, None], None, ),  # 1
)


class unsendSquareMessage_args(object):


    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('unsendSquareMessage_args')
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(unsendSquareMessage_args)
unsendSquareMessage_args.thrift_spec = (
)


class unsendSquareMessage_result(object):
    """
    Attributes:
     - success
     - e

    """


    def __init__(self, success=None, e=None,):
        self.success = success
        self.e = e

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.STRUCT:
                    self.success = UnsendMessageResponse()
                    self.success.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 1:
                if ftype == TType.STRUCT:
                    self.e = SquareException.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('unsendSquareMessage_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.STRUCT, 0)
            self.success.write(oprot)
            oprot.writeFieldEnd()
        if self.e is not None:
            oprot.writeFieldBegin('e', TType.STRUCT, 1)
            self.e.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(unsendSquareMessage_result)
unsendSquareMessage_result.thrift_spec = (
    (0, TType.STRUCT, 'success', [UnsendMessageResponse, None], None, ),  # 0
    (1, TType.STRUCT, 'e', [SquareException, None], None, ),  # 1
)


class fetchMyEvents_args(object):


    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('fetchMyEvents_args')
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(fetchMyEvents_args)
fetchMyEvents_args.thrift_spec = (
)


class fetchMyEvents_result(object):
    """
    Attributes:
     - success
     - e

    """


    def __init__(self, success=None, e=None,):
        self.success = success
        self.e = e

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.STRUCT:
                    self.success = FetchMyEventsResponse()
                    self.success.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 1:
                if ftype == TType.STRUCT:
                    self.e = SquareException.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('fetchMyEvents_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.STRUCT, 0)
            self.success.write(oprot)
            oprot.writeFieldEnd()
        if self.e is not None:
            oprot.writeFieldBegin('e', TType.STRUCT, 1)
            self.e.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(fetchMyEvents_result)
fetchMyEvents_result.thrift_spec = (
    (0, TType.STRUCT, 'success', [FetchMyEventsResponse, None], None, ),  # 0
    (1, TType.STRUCT, 'e', [SquareException, None], None, ),  # 1
)


class getSquareEmid_args(object):


    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('getSquareEmid_args')
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(getSquareEmid_args)
getSquareEmid_args.thrift_spec = (
)


class getSquareEmid_result(object):
    """
    Attributes:
     - success
     - e

    """


    def __init__(self, success=None, e=None,):
        self.success = success
        self.e = e

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.STRUCT:
                    self.success = GetSquareEmidResponse()
                    self.success.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 1:
                if ftype == TType.STRUCT:
                    self.e = SquareException.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('getSquareEmid_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.STRUCT, 0)
            self.success.write(oprot)
            oprot.writeFieldEnd()
        if self.e is not None:
            oprot.writeFieldBegin('e', TType.STRUCT, 1)
            self.e.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(getSquareEmid_result)
getSquareEmid_result.thrift_spec = (
    (0, TType.STRUCT, 'success', [GetSquareEmidResponse, None], None, ),  # 0
    (1, TType.STRUCT, 'e', [SquareException, None], None, ),  # 1
)


class getSquareMembersBySquare_args(object):


    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('getSquareMembersBySquare_args')
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(getSquareMembersBySquare_args)
getSquareMembersBySquare_args.thrift_spec = (
)


class getSquareMembersBySquare_result(object):
    """
    Attributes:
     - success
     - e

    """


    def __init__(self, success=None, e=None,):
        self.success = success
        self.e = e

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.STRUCT:
                    self.success = GetSquareMembersBySquareResponse()
                    self.success.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 1:
                if ftype == TType.STRUCT:
                    self.e = SquareException.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('getSquareMembersBySquare_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.STRUCT, 0)
            self.success.write(oprot)
            oprot.writeFieldEnd()
        if self.e is not None:
            oprot.writeFieldBegin('e', TType.STRUCT, 1)
            self.e.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(getSquareMembersBySquare_result)
getSquareMembersBySquare_result.thrift_spec = (
    (0, TType.STRUCT, 'success', [GetSquareMembersBySquareResponse, None], None, ),  # 0
    (1, TType.STRUCT, 'e', [SquareException, None], None, ),  # 1
)


class manualRepair_args(object):


    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('manualRepair_args')
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(manualRepair_args)
manualRepair_args.thrift_spec = (
)


class manualRepair_result(object):
    """
    Attributes:
     - success
     - e

    """


    def __init__(self, success=None, e=None,):
        self.success = success
        self.e = e

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.STRUCT:
                    self.success = ManualRepairResponse()
                    self.success.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 1:
                if ftype == TType.STRUCT:
                    self.e = SquareException.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('manualRepair_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.STRUCT, 0)
            self.success.write(oprot)
            oprot.writeFieldEnd()
        if self.e is not None:
            oprot.writeFieldBegin('e', TType.STRUCT, 1)
            self.e.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(manualRepair_result)
manualRepair_result.thrift_spec = (
    (0, TType.STRUCT, 'success', [ManualRepairResponse, None], None, ),  # 0
    (1, TType.STRUCT, 'e', [SquareException, None], None, ),  # 1
)


class inviteIntoSquareChat_args(object):


    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('inviteIntoSquareChat_args')
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(inviteIntoSquareChat_args)
inviteIntoSquareChat_args.thrift_spec = (
)


class inviteIntoSquareChat_result(object):
    """
    Attributes:
     - success
     - e

    """


    def __init__(self, success=None, e=None,):
        self.success = success
        self.e = e

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.STRUCT:
                    self.success = InviteIntoSquareChatResponse()
                    self.success.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 1:
                if ftype == TType.STRUCT:
                    self.e = SquareException.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('inviteIntoSquareChat_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.STRUCT, 0)
            self.success.write(oprot)
            oprot.writeFieldEnd()
        if self.e is not None:
            oprot.writeFieldBegin('e', TType.STRUCT, 1)
            self.e.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(inviteIntoSquareChat_result)
inviteIntoSquareChat_result.thrift_spec = (
    (0, TType.STRUCT, 'success', [InviteIntoSquareChatResponse, None], None, ),  # 0
    (1, TType.STRUCT, 'e', [SquareException, None], None, ),  # 1
)


class inviteToSquare_args(object):


    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('inviteToSquare_args')
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(inviteToSquare_args)
inviteToSquare_args.thrift_spec = (
)


class inviteToSquare_result(object):
    """
    Attributes:
     - success
     - e

    """


    def __init__(self, success=None, e=None,):
        self.success = success
        self.e = e

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.STRUCT:
                    self.success = InviteToSquareResponse()
                    self.success.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 1:
                if ftype == TType.STRUCT:
                    self.e = SquareException.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('inviteToSquare_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.STRUCT, 0)
            self.success.write(oprot)
            oprot.writeFieldEnd()
        if self.e is not None:
            oprot.writeFieldBegin('e', TType.STRUCT, 1)
            self.e.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(inviteToSquare_result)
inviteToSquare_result.thrift_spec = (
    (0, TType.STRUCT, 'success', [InviteToSquareResponse, None], None, ),  # 0
    (1, TType.STRUCT, 'e', [SquareException, None], None, ),  # 1
)


class getJoinedSquares_args(object):


    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('getJoinedSquares_args')
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(getJoinedSquares_args)
getJoinedSquares_args.thrift_spec = (
)


class getJoinedSquares_result(object):
    """
    Attributes:
     - success
     - e

    """


    def __init__(self, success=None, e=None,):
        self.success = success
        self.e = e

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.STRUCT:
                    self.success = GetJoinedSquaresResponse()
                    self.success.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 1:
                if ftype == TType.STRUCT:
                    self.e = SquareException.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('getJoinedSquares_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.STRUCT, 0)
            self.success.write(oprot)
            oprot.writeFieldEnd()
        if self.e is not None:
            oprot.writeFieldBegin('e', TType.STRUCT, 1)
            self.e.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(getJoinedSquares_result)
getJoinedSquares_result.thrift_spec = (
    (0, TType.STRUCT, 'success', [GetJoinedSquaresResponse, None], None, ),  # 0
    (1, TType.STRUCT, 'e', [SquareException, None], None, ),  # 1
)


class markAsRead_args(object):


    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('markAsRead_args')
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(markAsRead_args)
markAsRead_args.thrift_spec = (
)


class markAsRead_result(object):
    """
    Attributes:
     - success
     - e

    """


    def __init__(self, success=None, e=None,):
        self.success = success
        self.e = e

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.STRUCT:
                    self.success = MarkAsReadResponse()
                    self.success.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 1:
                if ftype == TType.STRUCT:
                    self.e = SquareException.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('markAsRead_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.STRUCT, 0)
            self.success.write(oprot)
            oprot.writeFieldEnd()
        if self.e is not None:
            oprot.writeFieldBegin('e', TType.STRUCT, 1)
            self.e.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(markAsRead_result)
markAsRead_result.thrift_spec = (
    (0, TType.STRUCT, 'success', [MarkAsReadResponse, None], None, ),  # 0
    (1, TType.STRUCT, 'e', [SquareException, None], None, ),  # 1
)


class reactToMessage_args(object):


    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('reactToMessage_args')
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(reactToMessage_args)
reactToMessage_args.thrift_spec = (
)


class reactToMessage_result(object):
    """
    Attributes:
     - success
     - e

    """


    def __init__(self, success=None, e=None,):
        self.success = success
        self.e = e

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.STRUCT:
                    self.success = ReactToMessageResponse()
                    self.success.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 1:
                if ftype == TType.STRUCT:
                    self.e = SquareException.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('reactToMessage_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.STRUCT, 0)
            self.success.write(oprot)
            oprot.writeFieldEnd()
        if self.e is not None:
            oprot.writeFieldBegin('e', TType.STRUCT, 1)
            self.e.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(reactToMessage_result)
reactToMessage_result.thrift_spec = (
    (0, TType.STRUCT, 'success', [ReactToMessageResponse, None], None, ),  # 0
    (1, TType.STRUCT, 'e', [SquareException, None], None, ),  # 1
)


class findSquareByInvitationTicket_args(object):


    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('findSquareByInvitationTicket_args')
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(findSquareByInvitationTicket_args)
findSquareByInvitationTicket_args.thrift_spec = (
)


class findSquareByInvitationTicket_result(object):
    """
    Attributes:
     - success
     - e

    """


    def __init__(self, success=None, e=None,):
        self.success = success
        self.e = e

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.STRUCT:
                    self.success = FindSquareByInvitationTicketResponse()
                    self.success.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 1:
                if ftype == TType.STRUCT:
                    self.e = SquareException.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('findSquareByInvitationTicket_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.STRUCT, 0)
            self.success.write(oprot)
            oprot.writeFieldEnd()
        if self.e is not None:
            oprot.writeFieldBegin('e', TType.STRUCT, 1)
            self.e.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(findSquareByInvitationTicket_result)
findSquareByInvitationTicket_result.thrift_spec = (
    (0, TType.STRUCT, 'success', [FindSquareByInvitationTicketResponse, None], None, ),  # 0
    (1, TType.STRUCT, 'e', [SquareException, None], None, ),  # 1
)


class fetchSquareChatEvents_args(object):


    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('fetchSquareChatEvents_args')
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(fetchSquareChatEvents_args)
fetchSquareChatEvents_args.thrift_spec = (
)


class fetchSquareChatEvents_result(object):
    """
    Attributes:
     - success
     - e

    """


    def __init__(self, success=None, e=None,):
        self.success = success
        self.e = e

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.STRUCT:
                    self.success = FetchSquareChatEventsResponse()
                    self.success.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 1:
                if ftype == TType.STRUCT:
                    self.e = SquareException.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('fetchSquareChatEvents_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.STRUCT, 0)
            self.success.write(oprot)
            oprot.writeFieldEnd()
        if self.e is not None:
            oprot.writeFieldBegin('e', TType.STRUCT, 1)
            self.e.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(fetchSquareChatEvents_result)
fetchSquareChatEvents_result.thrift_spec = (
    (0, TType.STRUCT, 'success', [FetchSquareChatEventsResponse, None], None, ),  # 0
    (1, TType.STRUCT, 'e', [SquareException, None], None, ),  # 1
)


class getSquare_args(object):


    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('getSquare_args')
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(getSquare_args)
getSquare_args.thrift_spec = (
)


class getSquare_result(object):
    """
    Attributes:
     - success
     - e

    """


    def __init__(self, success=None, e=None,):
        self.success = success
        self.e = e

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.STRUCT:
                    self.success = GetSquareResponse()
                    self.success.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 1:
                if ftype == TType.STRUCT:
                    self.e = SquareException.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('getSquare_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.STRUCT, 0)
            self.success.write(oprot)
            oprot.writeFieldEnd()
        if self.e is not None:
            oprot.writeFieldBegin('e', TType.STRUCT, 1)
            self.e.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(getSquare_result)
getSquare_result.thrift_spec = (
    (0, TType.STRUCT, 'success', [GetSquareResponse, None], None, ),  # 0
    (1, TType.STRUCT, 'e', [SquareException, None], None, ),  # 1
)


class getJoinableSquareChats_args(object):


    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('getJoinableSquareChats_args')
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(getJoinableSquareChats_args)
getJoinableSquareChats_args.thrift_spec = (
)


class getJoinableSquareChats_result(object):
    """
    Attributes:
     - success
     - e

    """


    def __init__(self, success=None, e=None,):
        self.success = success
        self.e = e

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.STRUCT:
                    self.success = GetJoinableSquareChatsResponse()
                    self.success.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 1:
                if ftype == TType.STRUCT:
                    self.e = SquareException.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('getJoinableSquareChats_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.STRUCT, 0)
            self.success.write(oprot)
            oprot.writeFieldEnd()
        if self.e is not None:
            oprot.writeFieldBegin('e', TType.STRUCT, 1)
            self.e.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(getJoinableSquareChats_result)
getJoinableSquareChats_result.thrift_spec = (
    (0, TType.STRUCT, 'success', [GetJoinableSquareChatsResponse, None], None, ),  # 0
    (1, TType.STRUCT, 'e', [SquareException, None], None, ),  # 1
)


class createSquare_args(object):


    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('createSquare_args')
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(createSquare_args)
createSquare_args.thrift_spec = (
)


class createSquare_result(object):
    """
    Attributes:
     - success
     - e

    """


    def __init__(self, success=None, e=None,):
        self.success = success
        self.e = e

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.STRUCT:
                    self.success = CreateSquareResponse()
                    self.success.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 1:
                if ftype == TType.STRUCT:
                    self.e = SquareException.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('createSquare_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.STRUCT, 0)
            self.success.write(oprot)
            oprot.writeFieldEnd()
        if self.e is not None:
            oprot.writeFieldBegin('e', TType.STRUCT, 1)
            self.e.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(createSquare_result)
createSquare_result.thrift_spec = (
    (0, TType.STRUCT, 'success', [CreateSquareResponse, None], None, ),  # 0
    (1, TType.STRUCT, 'e', [SquareException, None], None, ),  # 1
)


class getSquareChatAnnouncements_args(object):


    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('getSquareChatAnnouncements_args')
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(getSquareChatAnnouncements_args)
getSquareChatAnnouncements_args.thrift_spec = (
)


class getSquareChatAnnouncements_result(object):
    """
    Attributes:
     - success
     - e

    """


    def __init__(self, success=None, e=None,):
        self.success = success
        self.e = e

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.STRUCT:
                    self.success = GetSquareChatAnnouncementsResponse()
                    self.success.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 1:
                if ftype == TType.STRUCT:
                    self.e = SquareException.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('getSquareChatAnnouncements_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.STRUCT, 0)
            self.success.write(oprot)
            oprot.writeFieldEnd()
        if self.e is not None:
            oprot.writeFieldBegin('e', TType.STRUCT, 1)
            self.e.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(getSquareChatAnnouncements_result)
getSquareChatAnnouncements_result.thrift_spec = (
    (0, TType.STRUCT, 'success', [GetSquareChatAnnouncementsResponse, None], None, ),  # 0
    (1, TType.STRUCT, 'e', [SquareException, None], None, ),  # 1
)
fix_spec(all_structs)
del all_structs



# -*- coding: utf-8 -*-
import re


class Config(object):
    LINE_HOST_DOMAIN = "https://ga2.line.naver.jp"
    LINE_LEGY_HOST_DOMAIN = "https://legy-jp.line.naver.jp"
    LINE_GW_HOST_DOMAIN = "https://gwz.line.naver.jp"
    LINE_GF_HOST_DOMAIN = "https://gf.line.naver.jp"
    LINE_GD2_HOST_DOMAIN = "https://gd2.line.naver.jp"
    LINE_GD2K_HOST_DOMAIN = "https://gd2k.line.naver.jp"
    LINE_GD2S_HOST_DOMAIN = "https://gd2s.line.naver.jp"
    LINE_GD2G_HOST_DOMAIN = "https://gd2g.line.naver.jp"
    LINE_GD2U_HOST_DOMAIN = "https://gd2u.line.naver.jp"
    LINE_GD2V_HOST_DOMAIN = "https://gd2v.line.naver.jp"
    LINE_GD2I_HOST_DOMAIN = "https://gd2i.line.naver.jp"
    LINE_GD2W_HOST_DOMAIN = "https://gd2w.line.naver.jp"

    LINE_OBS_DOMAIN = "https://obs.line-apps.com"

    LINE_OBS_CDN_DOMAIN = "https://obs.line-scdn.net"
    LINE_PROFILE_CDN_DOMAIN = "https://profile.line-scdn.net"
    LINE_SHOP_CDN_DOMAIN = "https://shop.line-scdn.net"
    LINE_STICKERSHOP_CDN_DOMAIN = "https://stickershop.line-scdn.net"
    LINE_ECBOT_OBJ_CDN_DOMAIN = "https://ecbot-object.line-scdn.net"  # LINE購物護照
    LINE_SEARCH_CDN_DOMAIN = "https://search.line-scdn.net"
    LINE_STATIC_CDN_DOMAIN = "https://static.line-scdn.net"
    LINE_VOS_CDN_DOMAIN = "https://vos.line-scdn.net"
    LINE_SHOPPING_CDN_DOMAIN = "https://shopping.line-scdn.net"
    
    LINE_LAN_DOMAIN = "https://lan.line.me"

    LINE_LEGY_BETA_DOMAIN = "https://legy-beta.line-apps-beta.com"
    LINE_OBS_BETA_DOMAIN = "https://obs-beta.line-apps.com"
    LINE_OBS_CDN_BETA_DOMAIN = "https://cdn-obs-beta.line-apps.com/line"
    LINE_SHOP_DL_BETA_DOMAIN = "https://dl.shop.line.beta.naver.jp"
    LINE_STICKERSHOP_DL_BETA_DOMAIN = "https://dl.stickershop.line.beta.naver.jp"
    LINE_LAN_BETA_DOMAIN = "https://lan.line-beta.me"
    LINE_NOTICE_CDN_DOMAIN = "https://notice.line-beta.me"
    LINE_SCDN_APP_DOMAIN = "https://scdn.line-apps.com"
    LINE_YUKI_CDN_APP_DOMAIN = "https://yuki-cdn.line-apps.com"
    LINE_UTRACK_APP_DOMAIN = "https://uts-front.line-apps.com"

    LINE_LEGY_RC_DOMAIN = "https://legy-rc.line-apps-rc.com"
    LINE_OBS_RC_DOMAIN = "https://obs-rc.line-apps.com"
    LINE_OBS_CDN_RC_DOMAIN = "https://dl.common.line.naver.jp"

    LINE_NOTICE_DOMAIN = "https://notice.line.me"
    LINE_TV_DOMAIN = "https://tv.line.me"
    LINE_TV2_DOMAIN = "https://www.linetv.tw"
    LINE_STORE_DOMAIN = "https://store.line.me"
    LINE_MELODY_DOMAIN = "https://melody.line.me"
    LINE_POD_GAME_DOMAIN = "https://pod.game.line.me"
    LINE_WEBTOONS_DOMAIN = "https://www.webtoons.com"
    LINE_LINEFRIENDS_STORE_DOMAIN = "https://store.linefriends.com"
    LINE_LINEFRIENDS_TW_DOMAIN = "https://www.linefriends.com.tw"
    LINE_LINEFRIENDS_JP_DOMAIN = "https://www.linefriends.jp"
    LINE_SHOP_DOMAIN = "https://shop.line.me"
    LINE_BUY_DOMAIN = "https://buy.line.me"
    LINE_TODAY_DOMAIN = "https://today.line.me"
    LINE_TRAVEL_DOMAIN = "https://travel.line.me"
    LINE_FACTCHECKER_DOMAIN = "https://fact-checker.line.me"
    LINE_MUSIC_DOMAIN = "https://music.line.me"
    LINE_LIVE_DOMAIN = "https://live.line.me"
    LINE_MANGA_DOMAIN = "https://manga.line.me"
    LINE_EVENT_DOMAIN = "https://event.line.me"
    LINE_SEARCH_DOMAIN = "https://search.line.me"

    LINE_ENCRYPTION_ENDPOINT = "/enc"
    LINE_AGE_CHECK_ENDPOINT = "/ACS4"
    LINE_AUTH_ENDPOINT = "/RS3"
    LINE_AUTH_ENDPOINT_V4 = "/RS4"
    LINE_AUTH_EAP_ENDPOINT = "/ACCT/authfactor/eap/v1"
    LINE_BEACON_ENDPOINT = "/BEACON4"
    LINE_BUDDY_ENDPOINT = "/BUDDY3"
    LINE_CALL_ENDPOINT = "/V3"
    LINE_CANCEL_LONGPOLLING_ENDPOINT = "/CP4"
    LINE_CHANNEL_ENDPOINT = "/CH3"
    LINE_CHANNEL_ENDPOINT_V4 = "/CH4"
    LINE_PERSONAL_ENDPOINT_V4 = "/PS4"
    LINE_CHAT_APP_ENDPOINT = "/CAPP1"
    LINE_COIN_ENDPOINT = "/COIN4"
    LINE_COMPACT_E2EE_MESSAGE_ENDPOINT = "/ECA5"
    LINE_COMPACT_MESSAGE_ENDPOINT = "/C5"
    LINE_COMPACT_PLAIN_MESSAGE_ENDPOINT = "/CA5"
    LINE_CONN_INFO_ENDPOINT = "/R2"
    LINE_EXTERNAL_INTERLOCK_ENDPOINT = "/EIS4"
    LINE_IOT_ENDPOINT = "/IOT1"
    LINE_LIFF_ENDPOINT = "/LIFF1"
    LINE_NORMAL_ENDPOINT = "/S3"
    LINE_SECONDARY_QR_LOGIN_ENDPOINT = "/acct/lgn/sq/v1"
    LINE_SHOP_ENDPOINT = "/SHOP3"
    LINE_SHOP_AUTH_ENDPOINT = "/SHOPA"
    LINE_SNS_ADAPTER_ENDPOINT = "/SA4"
    LINE_SNS_ADAPTER_REGISTRATION_ENDPOINT = "/api/v4p/sa"
    LINE_SQUARE_ENDPOINT = "/SQ1"
    LINE_SQUARE_BOT_ENDPOINT = "/BP1"
    LINE_UNIFIED_SHOP_ENDPOINT = "/TSHOP4"
    LINE_WALLET_ENDPOINT = "/WALLET4"
    LINE_SECONDARY_PWLESS_LOGIN_ENDPOINT = "/acct/lgn/secpwless/v1"
    LINE_SECONDARY_PWLESS_LOGIN_PERMIT_ENDPOINT = "/acct/lp/lgn/secpwless/v1"
    LINE_SECONDARY_AUTH_FACTOR_PIN_CODE_ENDPOINT = "/acct/authfactor/second/pincode/v1"
    LINE_PWLESS_CREDENTIAL_MANAGEMENT_ENDPOINT = "/acct/authfactor/pwless/manage/v1"
    LINE_PWLESS_PRIMARY_REGISTRATION_ENDPOINT = "/ACCT/authfactor/pwless/v1"
    LINE_VOIP_GROUP_CALL_YOUTUBE_ENDPOINT = "/EXT/groupcall/youtube-api"
    LINE_E2EE_KEY_BACKUP_ENDPOINT = "/EKBS4"
    SECONDARY_DEVICE_LOGIN_VERIFY_PIN_WITH_E2EE = "/LF1"
    SECONDARY_DEVICE_LOGIN_VERIFY_PIN = "/Q"
    LINE_NOTIFY_SLEEP_ENDPOINT = "/F4"

    LINE_LAN_ANDROID_URL = "https://lan.line.me/v1/line/android/notification"
    LINE_LAN_IOS_URL = "https://lan.line.me/v1/line/ios/notification"
    LINE_MUSIC_WEBAPP_URL = "https://music.line.me/webapp"
    LINE_AMP_LOG_URL = "https://log1-amp.line-apps.com/log"
    LINE_SECONDYARY_REGISTER_API_URL = "https://w.line.me/lrs/r"

    LINE_LANGUAGE = "th_TH"
    LINE_SERVICE_REGION = "TH"

    APP_TYPE = "CHROMEOS"
    APP_VER = "11.19.2"
    SYSTEM_NAME = "DeachSword"
    SYSTEM_VER = "12.1.4"
    IP_ADDR = "8.8.8.8"
    EMAIL_REGEX = re.compile(r"[^@]+@[^@]+\.[^@]+")
    CONSENT_CHANNEL_ID_REGEX = re.compile(
        r"<input type=\"hidden\" name=\"channelId\" value=\"([^\"]+)\""
    )
    CONSENT_CSRF_TOKEN_REGEX = re.compile(
        r"<input type=\"hidden\" name=\"__csrf\" id=\"__csrf\" value=\"([^\"]+)\""
    )

    TOKEN_V3_SUPPORT = ["DESKTOPWIN", "DESKTOPMAC"]
    SYNC_SUPPORT = ["IOS", "IOSIPAD", "ANDROID"]

    def __init__(self, type="CHROME"):
        self.APP_NAME = None
        self.DEVICE_TYPE = type
        self.isSecondary = False
        if type == "DESKTOPWIN":
            self.APP_VER = "7.16.1.3000"
            self.SYSTEM_NAME = "WINDOWS"
            self.SYSTEM_VER = "10.0.0-NT-x64"
        elif type == "DESKTOPMAC":
            self.APP_VER = "7.16.1.3000"
            self.SYSTEM_NAME = "MAC"
        elif type == "CHROMEOS":
            self.APP_VER = "2.5.11"
            self.SYSTEM_NAME = "Chrome OS"
            self.SYSTEM_VER = "1"
        # elif type == "ANDROIDLITE":
        # self.APP_VER = "2.17.1"
        # self.SYSTEM_NAME = "Android OS"
        # self.isSecondary = True
        elif type in ["ANDROID", "ANDROIDSECONDARY"]:
            self.APP_VER = "13.4.1"
            self.SYSTEM_NAME = "Android OS"
        elif type == "IOS":
            self.APP_VER = "13.3.0"
            self.SYSTEM_NAME = "iOS"
        elif type == "IOSIPAD":
            self.APP_VER = "13.3.0"
            self.SYSTEM_NAME = "iOS"
        elif type == "WATCHOS":
            self.APP_VER = "13.3.0"
            self.SYSTEM_NAME = "Watch OS"
        elif type == "WEAROS":
            self.APP_VER = "13.4.1"
            self.SYSTEM_NAME = "Wear OS"
        elif type == "OPENCHAT_PLUS":
            pass
        elif type == "CHANNELGW":
            pass
        elif type == "CHANNELCP":
            pass
        elif type == "CLOVAFRIENDS":
            pass
        elif type == "BOT":
            pass
        elif type == "WAP":
            pass
        elif type == "WEB":
            pass
        elif type == "BIZWEB":
            pass
        elif type == "DUMMYPRIMARY":
            pass
        elif type == "SQUARE":
            pass
        elif type == "FIREFOXOS":
            pass
        elif type == "TIZEN":
            pass
        elif type == "VIRTUAL":
            pass
        elif type == "CHRONO":
            pass
        elif type == "WINMETRO":
            pass
        elif type == "S40":
            pass
        elif type == "WINPHONE":
            pass
        elif type == "BLACKBERRY":
            pass
        elif type == "INTERNAL":
            pass
        else:
            raise Exception(f"未知的Device , 請至 config.py 新增")
        self.APP_TYPE = type
        self.USER_AGENT = "Line/%s" % self.APP_VER

    def initAppConfig(self, app_type, app_version, os_name, os_version):
        self.APP_TYPE = "CHROMEOS"
        if app_type is not None:
            self.APP_TYPE = app_type
        if app_version is not None:
            self.APP_VER = app_version
        if os_name is not None:
            self.SYSTEM_NAME = os_name
        if os_version is not None:
            self.SYSTEM_VER = os_version
        self.APP_NAME = "%s\t%s\t%s\t%s" % (
            self.APP_TYPE,
            self.APP_VER,
            self.SYSTEM_NAME,
            self.SYSTEM_VER,
        )
        if self.isSecondary:
            self.APP_NAME += ";SECONDARY"



# -*- coding: utf-8 -*-


class HomeSafetyCheckService(object):
    HomeSafetyCheckService_REQ_TYPE = 3
    HomeSafetyCheckService_RES_TYPE = 3
    HomeSafetyCheckService_API_PATH = "/EXT/home/safety-check/safety-check"

    def __init__(self):
        pass

    def deleteSafetyStatus(self, disasterId: str):
        METHOD_NAME = "deleteSafetyStatus"
        params = [[12, 1, [[11, 1, disasterId]]]]
        sqrd = self.generateDummyProtocol(METHOD_NAME, params,
                                          self.HomeSafetyCheckService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(
            self.HomeSafetyCheckService_API_PATH, sqrd,
            self.HomeSafetyCheckService_RES_TYPE)

    def getDisasterCases(self):
        METHOD_NAME = "getDisasterCases"
        params = [[12, 1, []]]
        sqrd = self.generateDummyProtocol(METHOD_NAME, params,
                                          self.HomeSafetyCheckService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(
            self.HomeSafetyCheckService_API_PATH,
            sqrd,
            self.HomeSafetyCheckService_RES_TYPE,
            readWith=f"{__class__.__name__}.{METHOD_NAME}")

    def updateSafetyStatus(self,
                           disasterId: str,
                           message: str,
                           safetyStatus: int = 1):
        """
        - safetyStatus:
            SAFE(1),
            NOT_SAFE(2);
        """
        METHOD_NAME = "updateSafetyStatus"
        params = [[
            12, 1,
            [
                [11, 1, disasterId],
                [8, 2, safetyStatus],
                [11, 3, message],
            ]
        ]]
        sqrd = self.generateDummyProtocol(METHOD_NAME, params,
                                          self.HomeSafetyCheckService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(
            self.HomeSafetyCheckService_API_PATH, sqrd,
            self.HomeSafetyCheckService_RES_TYPE)



# -*- coding: utf-8 -*-
from typing import List
from typing import TYPE_CHECKING

from .BaseService import BaseService, BaseServiceStruct
from .SquareService import SquareService

if TYPE_CHECKING:
    from CHRLINE import CHRLINE


class SquareLiveTalkService(BaseService):
    SquareLiveTalkService_REQ_TYPE = 4  # BASE_VAL
    SquareLiveTalkService_RES_TYPE = 4  # BASE_VAL
    SquareLiveTalkService_API_PATH = None  # BASE_VAL

    def __init__(self):
        self.SquareLiveTalkService_API_PATH = "/SQLV1"

    def acceptSpeakers(self, squareChatMid: str, sessionId: str, targetMids: List[str]):
        """Accept speakers."""
        METHOD_NAME = "acceptSpeakers"
        params = [[11, 1, squareChatMid], [11, 2, sessionId], [14, 3, [11, targetMids]]]
        return SquareLiveTalkServiceStruct.SendRequestByName(self, METHOD_NAME, params)

    def acceptToChangeRole(
        self, squareChatMid: str, sessionId: str, inviteRequestId: str
    ):
        """Accept to change role."""
        METHOD_NAME = "acceptToChangeRole"
        params = [[11, 1, squareChatMid], [11, 2, sessionId], [11, 3, inviteRequestId]]
        return SquareLiveTalkServiceStruct.SendRequestByName(self, METHOD_NAME, params)

    def acceptToListen(self, squareChatMid: str, sessionId: str, inviteRequestId: str):
        """Accept to listen."""
        METHOD_NAME = "acceptToListen"
        params = [[11, 1, squareChatMid], [11, 2, sessionId], [11, 3, inviteRequestId]]
        return SquareLiveTalkServiceStruct.SendRequestByName(self, METHOD_NAME, params)

    def acceptToSpeak(self, squareChatMid: str, sessionId: str, inviteRequestId: str):
        """Accept to speak."""
        METHOD_NAME = "acceptToSpeak"
        params = [[11, 1, squareChatMid], [11, 2, sessionId], [11, 3, inviteRequestId]]
        return SquareLiveTalkServiceStruct.SendRequestByName(self, METHOD_NAME, params)

    def cancelToSpeak(self, squareChatMid: str, sessionId: str):
        """Cancel to speak."""
        METHOD_NAME = "cancelToSpeak"
        params = [[11, 1, squareChatMid], [11, 2, sessionId]]
        return SquareLiveTalkServiceStruct.SendRequestByName(self, METHOD_NAME, params)

    def endLiveTalk(self, squareChatMid: str, sessionId: str):
        """End live talk."""
        METHOD_NAME = "endLiveTalk"
        params = [[11, 1, squareChatMid], [11, 2, sessionId]]
        return SquareLiveTalkServiceStruct.SendRequestByName(self, METHOD_NAME, params)

    def fetchLiveTalkEvents(
        self, squareChatMid: str, sessionId: str, syncToken: str, limit: int = 50
    ):
        """Fetch live talk events."""
        METHOD_NAME = "fetchLiveTalkEvents"
        params = [
            [11, 1, squareChatMid],
            [11, 2, sessionId],
            [11, 3, syncToken],
            [8, 4, limit],
        ]
        return SquareLiveTalkServiceStruct.SendRequestByName(self, METHOD_NAME, params)

    def findLiveTalkByInvitationTicket(self, invitationTicket: str):
        """Find live talk by invitation ticket."""
        METHOD_NAME = "findLiveTalkByInvitationTicket"
        params = [[11, 1, invitationTicket]]
        return SquareLiveTalkServiceStruct.SendRequestByName(self, METHOD_NAME, params)

    def forceEndLiveTalk(self, squareChatMid: str, sessionId: str):
        """Force end live talk."""
        METHOD_NAME = "forceEndLiveTalk"
        params = [[11, 1, squareChatMid], [11, 2, sessionId]]
        return SquareLiveTalkServiceStruct.SendRequestByName(self, METHOD_NAME, params)

    def getLiveTalkInfoForNonMember(
        self, squareChatMid: str, sessionId: str, speakers: List[str]
    ):
        """Get live talk info for non-member."""
        METHOD_NAME = "getLiveTalkInfoForNonMember"
        params = [[11, 1, squareChatMid], [11, 2, sessionId], [15, 3, [11, speakers]]]
        return SquareLiveTalkServiceStruct.SendRequestByName(self, METHOD_NAME, params)

    def getLiveTalkInvitationUrl(self, squareChatMid: str, sessionId: str):
        """Get live talk invitation url."""
        METHOD_NAME = "getLiveTalkInvitationUrl"
        params = [[11, 1, squareChatMid], [11, 2, sessionId]]
        return SquareLiveTalkServiceStruct.SendRequestByName(self, METHOD_NAME, params)

    def getLiveTalkSpeakersForNonMember(
        self, squareChatMid: str, sessionId: str, speakers: List[str]
    ):
        """Get live talk speakers for non-member."""
        METHOD_NAME = "getLiveTalkSpeakersForNonMember"
        params = [[11, 1, squareChatMid], [11, 2, sessionId], [15, 3, [11, speakers]]]
        return SquareLiveTalkServiceStruct.SendRequestByName(self, METHOD_NAME, params)

    def getSquareInfoByChatMid(self, squareChatMid: str):
        """Get square info by chat mid."""
        METHOD_NAME = "getSquareInfoByChatMid"
        params = [[11, 1, squareChatMid]]
        return SquareLiveTalkServiceStruct.SendRequestByName(self, METHOD_NAME, params)

    def inviteToChangeRole(
        self, squareChatMid: str, sessionId: str, targetMid: str, targetRole: int
    ):
        """
        Invite to change role.

        - targetRole:
            HOST(1),
            CO_HOST(2),
            GUEST(3);
        """
        METHOD_NAME = "inviteToChangeRole"
        params = [
            [11, 1, squareChatMid],
            [11, 2, sessionId],
            [11, 3, targetMid],
            [8, 4, targetRole],
        ]
        return SquareLiveTalkServiceStruct.SendRequestByName(self, METHOD_NAME, params)

    def inviteToListen(self, squareChatMid: str, sessionId: str, targetMid: str):
        """Invite to listen."""
        METHOD_NAME = "inviteToListen"
        params = [[11, 1, squareChatMid], [11, 2, sessionId], [11, 3, targetMid]]
        return SquareLiveTalkServiceStruct.SendRequestByName(self, METHOD_NAME, params)

    def inviteToLiveTalk(self, squareChatMid: str, sessionId: str, invitees: List[str]):
        """Invite to live talk."""
        METHOD_NAME = "inviteToLiveTalk"
        params = [[11, 1, squareChatMid], [11, 2, sessionId], [15, 3, [11, invitees]]]
        return SquareLiveTalkServiceStruct.SendRequestByName(self, METHOD_NAME, params)

    def inviteToSpeak(self, squareChatMid: str, sessionId: str, targetMid: str):
        """Invite to speak."""
        METHOD_NAME = "inviteToSpeak"
        params = [[11, 1, squareChatMid], [11, 2, sessionId], [11, 3, targetMid]]
        return SquareLiveTalkServiceStruct.SendRequestByName(self, METHOD_NAME, params)

    def joinLiveTalk(self, squareChatMid: str, sessionId: str, wantToSpeak: bool):
        """Join live talk."""
        METHOD_NAME = "joinLiveTalk"
        params = [[11, 1, squareChatMid], [11, 2, sessionId], [2, 3, wantToSpeak]]
        return SquareLiveTalkServiceStruct.SendRequestByName(self, METHOD_NAME, params)

    def kickOutLiveTalkParticipants(
        self, squareChatMid: str, sessionId: str, mid: str = None
    ):
        """Kick out live talk participants."""
        METHOD_NAME = "kickOutLiveTalkParticipants"
        target = []
        if mid is not None:
            liveTalkParticipant = [[11, 1, mid]]
            target.append([12, 1, liveTalkParticipant])
        else:
            target.append([12, 2, []])
        params = [[11, 1, squareChatMid], [11, 2, sessionId], [12, 3, target]]
        return SquareLiveTalkServiceStruct.SendRequestByName(self, METHOD_NAME, params)

    def rejectSpeakers(self, squareChatMid: str, sessionId: str, targetMids: List[str]):
        """Reject speakers."""
        METHOD_NAME = "rejectSpeakers"
        params = [[11, 1, squareChatMid], [11, 2, sessionId], [14, 3, [11, targetMids]]]
        return SquareLiveTalkServiceStruct.SendRequestByName(self, METHOD_NAME, params)

    def rejectToSpeak(self, squareChatMid: str, sessionId: str, inviteRequestId: str):
        """Reject to speak."""
        METHOD_NAME = "rejectToSpeak"
        params = [[11, 1, squareChatMid], [11, 2, sessionId], [11, 3, inviteRequestId]]
        return SquareLiveTalkServiceStruct.SendRequestByName(self, METHOD_NAME, params)

    def reportLiveTalk(self, squareChatMid: str, sessionId: str, reportType: int):
        """
        Report live talk.

        - reportType:
            ADVERTISING(1),
            GENDER_HARASSMENT(2),
            HARASSMENT(3),
            IRRELEVANT_CONTENT(4),
            OTHER(5);
        """
        METHOD_NAME = "reportLiveTalk"
        params = [[11, 1, squareChatMid], [11, 2, sessionId], [8, 3, reportType]]
        return SquareLiveTalkServiceStruct.SendRequestByName(self, METHOD_NAME, params)

    def reportLiveTalkSpeaker(
        self, squareChatMid: str, sessionId: str, speakerMemberMid: str, reportType: int
    ):
        """
        Report live talk speaks.

        - reportType:
            ADVERTISING(1),
            GENDER_HARASSMENT(2),
            HARASSMENT(3),
            IRRELEVANT_CONTENT(4),
            OTHER(5);
        """
        METHOD_NAME = "reportLiveTalkSpeaker"
        params = [
            [11, 1, squareChatMid],
            [11, 2, sessionId],
            [11, 3, speakerMemberMid],
            [8, 4, reportType],
        ]
        return SquareLiveTalkServiceStruct.SendRequestByName(self, METHOD_NAME, params)

    def requestToListen(self, squareChatMid: str, sessionId: str):
        """Request to listen."""
        METHOD_NAME = "requestToListen"
        params = [[11, 1, squareChatMid], [11, 2, sessionId]]
        return SquareLiveTalkServiceStruct.SendRequestByName(self, METHOD_NAME, params)

    def requestToSpeak(self, squareChatMid: str, sessionId: str):
        """Request to speak."""
        METHOD_NAME = "requestToSpeak"
        params = [[11, 1, squareChatMid], [11, 2, sessionId]]
        return SquareLiveTalkServiceStruct.SendRequestByName(self, METHOD_NAME, params)

    def startLiveTalk(
        self, squareChatMid: str, title: str, _type: int = 1, speakerSetting: int = 2
    ):
        """
        Start live talk.

        - type:
            PUBLIC(1),
            PRIVATE(2);
        - speakerSetting:
            LIMITED_SPEAKERS(1),
            ALL_AS_SPEAKERS(2);
        """
        METHOD_NAME = "startLiveTalk"
        params = [
            [11, 1, squareChatMid],
            [11, 2, title],
            [8, 3, _type],
            [8, 4, speakerSetting],
        ]
        return SquareLiveTalkServiceStruct.SendRequestByName(self, METHOD_NAME, params)

    def updateLiveTalkAttrs(
        self,
        squareChatMid: str,
        sessionId: str,
        updatedAttrs: List[int],
        title: str = None,
        speakerSetting: int = None,
        allowRequestToSpeak: bool = None,
    ):
        """
        Update live talk attrs.

        - updatedAttrs:
            TITLE(1),
            SPEAKER_SETTING(2),
            ALLOW_REQUEST_TO_SPEAK(3);
        """
        METHOD_NAME = "updateLiveTalkAttrs"
        liveTalk = [[11, 1, squareChatMid], [11, 2, sessionId]]
        if title is not None:
            liveTalk.append([11, 3, title])
        if speakerSetting is not None:
            liveTalk.append([8, 5, speakerSetting])
        if allowRequestToSpeak is not None:
            liveTalk.append([2, 6, allowRequestToSpeak])
        params = [[14, 1, [8, updatedAttrs]], [12, 2, liveTalk]]
        return SquareLiveTalkServiceStruct.SendRequestByName(self, METHOD_NAME, params)


class SquareLiveTalkServiceStruct(BaseServiceStruct):
    @staticmethod
    def SendRequestByName(client: "CHRLINE", name: str, request: str):
        payload = __class__.BaseRequest(request)
        sqrd = client.generateDummyProtocol(
            name, payload, client.SquareLiveTalkService_REQ_TYPE
        )
        return client.postPackDataAndGetUnpackRespData(
            client.SquareLiveTalkService_API_PATH,
            sqrd,
            client.SquareLiveTalkService_RES_TYPE,
            baseException=SquareService.SQUARE_EXCEPTION,
            readWith=f"SquareLiveTalkService.{name}",
        )
